
from vsg.rules import case_rule
from vsg import utils


class rule_009(case_rule):
    '''
    Process rule 009 checks the "process" keyword has proper case on the closing of a process.
    '''

    def __init__(self):
        case_rule.__init__(self, 'process', '009', 'isEndProcess')
        self.solution = 'Change "process" keyword to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['process'])




from vsg.rules import case_rule


class rule_016(case_rule):
    '''
    Instantiation rule 016 checks the generic name has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'instantiation', '016', 'isInstantiationGenericAssignment')
        self.case = 'upper'
        self.solution = 'Change generic name to '

    def _extract(self, oLine):
        ret = []
        if not oLine.isInstantiationGenericKeyword:
            ret.append(oLine.line.split()[0])

        return ret




from vsg.rules import line_above_rule


class rule_012(line_above_rule):
    '''
    Package rule 012 checks for a blank line above the "end package" keywords.
    '''

    def __init__(self):
        line_above_rule.__init__(self, 'package', '012', 'isPackageEnd')




from vsg import rule

import re


class rule_024(rule.rule):
    '''
    Architecture rule 024 checks for the architecture name in the "end architecture" statement.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'architecture', '024')
        self.solution = 'Add architecture name keyword.'
        self.phase = 1

    def _pre_analyze(self):
        self.dFix['label'] = {}
        self.sLabel = ""

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isArchitectureKeyword:
            self.sLabel = oLine.line.split()[1]
        if oLine.isEndArchitecture and not re.match('^\s*end\s+architecture\s+\w+', oLine.line, re.IGNORECASE):
            if re.match('^\s*end\s+architecture', oLine.line, re.IGNORECASE):
                self.add_violation(iLineNumber)
                self.dFix['label'][iLineNumber] = self.sLabel
            elif not re.match('^\s*end\s+\w+', oLine.line, re.IGNORECASE):
                self.add_violation(iLineNumber)
                self.dFix['label'][iLineNumber] = self.sLabel

    def _fix_violations(self, oFile):
        for iLineNumber, oLine in enumerate(oFile.lines):
            if iLineNumber in self.dFix['label']:
                sLine = oLine.line
                oLine.update_line(sLine.replace(';', ' ' + self.dFix['label'][iLineNumber].upper() + ';', 1))





class rule():

    def __init__(self, name=None, identifier=None):
        self.name = name
        self.identifier = identifier
        self.solution = None
        self.violations = []
        self.indentSize = 2
        self.phase = None
        self.subphase = 1
        self.disable = False
        self.fixable = True
        self.dFix = {}
        self.dFix['violations'] = {}
        self.configuration = ['indentSize', 'phase', 'disable', 'fixable']

    def configure(self, dConfiguration):
        '''Configures attributes on rules using a dictionary of the following form:

            dConfiguration['rule'] = {}
            dConfiguration['rule']['xyz_001'] = {}
            dConfiguration['rule']['xyz_001']['disable'] = True
            dConfiguration['rule']['xyz_001']['solution'] = 'This is the new solution'
            dConfiguration['rule']['xyz_002'] = {}
            dConfiguration['rule']['xyz_002']['disable'] = False
            dConfiguration['rule']['global'] = {}
            dConfiguration['rule']['global']['indentSize'] = 4

          The rule:global dictionary will apply to all rules.
          Individual rule attributes can be modified with [self.name_self.identifier].
        '''

        self._configure_global_rule_attributes(dConfiguration)
        self._configure_rule_attributes(dConfiguration)

    def report_violations(self, iLineNumber, sOutputFormat, sFileName, fQuiet=False):
        '''
        Reports any rule violations to stdout.
        '''
        for sViolation in self.violations:
            if str(sViolation).startswith(str(iLineNumber) + '-') or str(iLineNumber) == str(sViolation):
                if not fQuiet:
                    if sOutputFormat == 'vsg':
                        sOutputString = '  '
                        sOutputString += (self.name + '_' + self.identifier).ljust(25)
                        sOutputString += ' | '
                        sOutputString += str(sViolation).rjust(10)
                        sOutputString += ' | '
                        sOutputString += self._get_solution(iLineNumber)
                    else:
                        sOutputString = 'ERROR: '
                        sOutputString += sFileName
                        sOutputString += '('
                        sOutputString += str(iLineNumber)
                        sOutputString += ')'
                        sOutputString += self.name + '_' + self.identifier
                        sOutputString += ' -- '
                        sOutputString += self._get_solution(iLineNumber)
                    print(sOutputString)
                return 1
        return 0

    def fix(self, oFile):
        '''
        Applies fixes for any rule violations.
        '''
        if self.fixable:
            self.analyze(oFile)
            self._fix_violations(oFile)
            self.violations = []
            self.dFix = {}
            self.dFix['violations'] = {}

    def add_violation(self, lineNumber):
        '''
        Adds a linenumber to a violations list.
        '''
        if lineNumber not in self.violations:
            self.violations.append(lineNumber)

    def analyze(self, oFile):
        '''
        Performs the analysis.
        '''
        self._pre_analyze()
        for iLineNumber, oLine in enumerate(oFile.lines):
            if not self._is_vsg_off(oLine):
                self._analyze(oFile, oLine, iLineNumber)

    def _configure_global_rule_attributes(self, dConfiguration):
        '''
        Updates rule attributes based on configuration input files
        '''
        try:
            for sAttributeName in dConfiguration['rule']['global']:
                if sAttributeName in self.__dict__:
                    self.__dict__[sAttributeName] = dConfiguration['rule']['global'][sAttributeName]
        except KeyError:
            pass

    def _configure_rule_attributes(self, dConfiguration):
        '''
        Updates rule attributes based on configuration input files
        '''
        try:
            for sAttributeName in dConfiguration['rule'][self.name + '_' + self.identifier]:
                if sAttributeName in self.__dict__:
                    self.__dict__[sAttributeName] = dConfiguration['rule'][self.name + '_' + self.identifier][sAttributeName]
        except KeyError:
            pass

    def _is_vsg_off(self, oLine):
        '''
        Checks if the rule has been disabled for a given line.
        '''
        if 'vsg_off' in oLine.codeTags:
            if len(oLine.codeTags['vsg_off']) == 0 or self.name + '_' + self.identifier in oLine.codeTags['vsg_off']:
                return True
        return False

    def get_configuration(self):
        '''
        Returns a dictionary of every configurable attribute of the rule.
        '''
        dConfig = {}
        for sParameter in self.configuration:
            dConfig[sParameter] = getattr(self, sParameter)
        return dConfig

    def _get_solution(self, iLineNumber):
        '''
        By default this method return self.solution.
        This method can be overloaded by a rule if a more complex solution output is required.
        '''
        return self.solution

    def _pre_analyze(self):
        '''
        This method is called before the _analyze method and allows each rule to setup any variables needed.
        '''
        return




from vsg import rule
from vsg import utils

import re


class rule_023(rule.rule):
    '''If rule 023 checks the "elsif" keyword is on it's own line.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'if'
        self.identifier = '023'
        self.solution = 'Move "elsif" keyword to it\'s own line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isElseIfKeyword and not re.match('^\s*elsif', oLine.lineLower):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            utils.split_line_before_word(oFile, iLineNumber, 'elsif')
            oFile.lines[iLineNumber].isLastEndIf = False
            oFile.lines[iLineNumber].isElseIfKeyword = False
            oFile.lines[iLineNumber + 1].isIfKeyword = False
            oFile.lines[iLineNumber + 1].isFirstIf = False




from vsg.rules import case_rule
from vsg import check


class rule_010(case_rule):
    '''
    Package rule 010 checks the package name has proper case in the package declaration.
    '''

    def __init__(self):
        case_rule.__init__(self, 'package', '010', 'isPackageKeyword')
        self.solution = 'Change package name to '
        self.case = 'upper'

    def _extract(self, oLine):
        if not check.has_package_name(oLine):
            return []

        return [oLine.line.replace(';', '').split()[1]]




from vsg.rules import case_rule
from vsg import utils


class rule_031(case_rule):
    '''
    Instantiation rule 031 checks the "component" keyword has proper case if it is used.
    '''

    def __init__(self):
        case_rule.__init__(self, 'instantiation', '031', 'isInstantiationDeclaration')
        self.solution = 'Change "component" keyword to '
        self.disable = True

    def _extract(self, oLine):
        if not oLine.isDirectInstantiationDeclaration:
            return utils.extract_words(oLine, ['component'])
        else:
            return []




from vsg import rule

import re


class rule_013(rule.rule):
    '''Whitespace rule 013 checks for spaces before and after math operators.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'whitespace'
        self.identifier = '013'
        self.phase = 2
        self.solution = 'Add a single space before and/or after math operator.'

    def _analyze(self, oFile, oLine, iLineNumber):
        sLine = oLine.lineNoComment
        if re.match('^.*\)[and|nand|or|nor|xor|xnor]', sLine, flags=re.IGNORECASE):
            self.add_violation(iLineNumber)
        if re.match('^.* and\(', sLine, flags=re.IGNORECASE):
            self.add_violation(iLineNumber)
        if re.match('^.* nand\(', sLine, flags=re.IGNORECASE):
            self.add_violation(iLineNumber)
        if re.match('^.* or\(', sLine, flags=re.IGNORECASE):
            self.add_violation(iLineNumber)
        if re.match('^.* nor\(', sLine, flags=re.IGNORECASE):
            self.add_violation(iLineNumber)
        if re.match('^.* xor\(', sLine, flags=re.IGNORECASE):
            self.add_violation(iLineNumber)
        if re.match('^.* xnor\(', sLine, flags=re.IGNORECASE):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            iCommentIndex = oLine.line.find('--')
            if iCommentIndex == -1:
                oLine.update_line(re.sub(r'\)([and|nand|or|nor|xor|xnor])', r') \1', oLine.line, flags=re.IGNORECASE))
                oLine.update_line(re.sub(r'([and|nand|or|nor|xor|xnor])\(', r'\1 (', oLine.line, flags=re.IGNORECASE))




from vsg.rules import case_rule
from vsg import utils


class rule_006(case_rule):
    '''
    Package rule 006 checks the "end" and "package" keywords have proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'package', '006', 'isPackageEnd')
        self.solution = 'Change "end" and "package" keywords to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['end', 'package'])




from vsg.rules import line_below_rule


class rule_018(line_below_rule):
    '''
    Component rule 018 checks for a blank line below the
    "end component" keywords.
    '''

    def __init__(self):
        line_below_rule.__init__(self)
        self.name = 'component'
        self.identifier = '018'
        self.condition = 'isComponentEnd'




from vsg.rules import multiple_spaces_after_rule


class rule_003(multiple_spaces_after_rule):
    '''
    Constant rule 003 checks the number of spaces after the "constant" keyword.
    '''

    def __init__(self):
        multiple_spaces_after_rule.__init__(self, 'constant', '003', 'isConstant', 'constant')



import re


def component(dVars, oLine):
    '''
    Classifies component declarations.

    component identifier [is]
        [generic (generic_interface_list) ;]
        [port (port_interface_list) ;]
    end component [identifier] ;

    Sets the following line attributes:

      * isComponentDeclaration
      * insideComponent
      * indentLevel
      * isComponentEnd

    Modifies the following variables:

      * iCurrentIndentLevel
    '''
    # Check Component declarations
    if re.match('^\s*component', oLine.lineLower) and not oLine.insideComponent:
        oLine.isComponentDeclaration = True
        oLine.insideComponent = True
        oLine.indentLevel = 1
        dVars['iCurrentIndentLevel'] += 1

    if re.match('^\s*end\s+component', oLine.lineLower):
        oLine.isComponentEnd = True
        oLine.indentLevel = 1
        dVars['iCurrentIndentLevel'] -= 1



from .are_there_spaces_after import *
from .get_package_name import *
from .has_package_name import *
from .identifier_alignment import *
from .indent_of_comments_above import *
from .indent import *
from .is_blank_line_after import *
from .is_blank_line_before import *
from .is_lowercase import *
from .is_uppercase import *
from .is_no_blank_line_after import *
from .is_no_blank_line_before import *
from .is_single_space_after_character import *
from .is_single_space_after import *
from .is_single_space_before_character import *
from .is_single_space_before import *
from .is_uppercase import *
from .keyword_alignment import *
from .multiline_alignment import *
from .multiline_alignment_with_parenthesis import *
from .has_prefix import *
from .has_suffix import *



'''
This module contains functions for rules to perform their checks.
'''

import re


def indent(self, oLine, iLineNumber):
    '''
    Adds a violation if the indent of the line does not match the desired level.

    Parameters

      self: (rule object)

      oLine: (line object)

      iLineNumber: (integer)

    '''
    if not oLine.isBlank:
        try:
            if not re.match('^\s{' + str(self.indentSize * oLine.indentLevel) + '}\S', oLine.line):
                self.add_violation(iLineNumber)
        except TypeError:
            pass




from vsg.rules import case_rule
from vsg import utils


class rule_010(case_rule):
    '''
    Signal rule 010 checks the signal type has proper case if it is a VHDL keyword.
    '''

    def __init__(self):
        case_rule.__init__(self, 'signal', '010', 'isSignal')
        self.solution = 'Change signal type name to '
        self.disabled = True

    def _extract(self, oLine):
        return utils.extract_type_name_vhdl_only(oLine)




from vsg.rules import single_space_after_rule


class rule_013(single_space_after_rule):
    '''
    Generate rule 013 checks for a single space between "generate" keyword and the label.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'generate', '013', 'isGenerateEnd', 'generate')
        self.solution = 'Ensure there is only one space between the "generate" keyword and the label.'




from vsg.rules import remove_blank_lines_above_rule


class rule_024(remove_blank_lines_above_rule):
    '''
    Port rule 024 checks for blank lines above the closing parenthesis.
    '''

    def __init__(self):
        remove_blank_lines_above_rule.__init__(self, 'port', '024', 'isEndPortMap')
        self.solution = 'Remove blank lines above ); keyword.'




from vsg.rules import case_rule
from vsg import utils


class rule_006(case_rule):
    '''
    Instantiation rule 006 checks the "port map" keywords have proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'instantiation', '006', 'isInstantiationPortKeyword')
        self.solution = 'Change "port map" keywords to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['port', 'map'])



import re


def wait(dVars, oLine):

    if re.match('^\s*wait', oLine.lineLower):
        if not oLine.insideEntity and \
           not oLine.insideComponent and \
           not oLine.insideInstantiation:
            oLine.isWait = True
            oLine.indentLevel = dVars['iCurrentIndentLevel']




from vsg import rule
from vsg import utils

import re


class rule_019(rule.rule):
    '''
    Entity rule 019 checks for the entity name on the "end entity" statement.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'entity'
        self.identifier = '019'
        self.solution = 'Add the entity name.'
        self.phase = 1

    def _pre_analyze(self):
        self.sEntityName = ''

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isEntityDeclaration:
            self.sEntityName = utils.extract_entity_identifier(oLine)[0]
        if oLine.isEndEntityDeclaration and re.match('^\s*end\s+entity', oLine.line, re.IGNORECASE):
            if not re.match('^\s*end\s+entity\s+\w+', oLine.line, re.IGNORECASE):
                self.add_violation(iLineNumber)
                self.dFix['violations'][iLineNumber] = self.sEntityName

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            sLine = oLine.line
            iIndex = oLine.lineLower.find('entity') + len('entity')
            oLine.update_line(sLine[:iIndex] + ' ' + self.dFix['violations'][iLineNumber] + sLine[iIndex:])




from vsg import rule
from vsg import check
from vsg import fix


class rule_018(rule.rule):
    '''
    Instantiation rule 018 checks for a single space between map and (
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '018'
        self.solution = 'Ensure a single space exists between "map" and (.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isInstantiationGenericKeyword or oLine.isInstantiationPortKeyword:
            check.is_single_space_after(self, 'map', oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], 'map')




from vsg import rule
from vsg import utils

import re


class rule_021(rule.rule):
    '''
    If rule 021 checks the "else" keyword is on it's own line.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'if'
        self.identifier = '021'
        self.solution = 'Move "else" keyword to it\'s own line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isElseKeyword and not re.match('^\s*else', oLine.lineLower):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            utils.split_line_before_word(oFile, iLineNumber, 'else')
            oFile.lines[iLineNumber].isElseKeyword = False
            oFile.lines[iLineNumber + 1].isIfKeyword = False
            oFile.lines[iLineNumber + 1].isElseIfKeyword = False
            oFile.lines[iLineNumber + 1].isThenKeyword = False
            oFile.lines[iLineNumber].isLastEndIf = False
            oFile.lines[iLineNumber + 1].isFirstIf = False
            if not oFile.lines[iLineNumber].isIfKeyword:
                oFile.lines[iLineNumber + 1].indentLevel -= 1



import re


def block(self, dVars, oLine):

    classify_block_keyword(self, dVars, oLine)
    if oLine.insideBlock:
        classify_begin_keyword(dVars, oLine)
        classify_end_keyword(dVars, oLine)


def classify_block_keyword(self, dVars, oLine):
    if (re.match('^\s*block', oLine.lineNoComment) or re.match('^\s*\w+\s*:\s*block', oLine.lineNoComment)):
        oLine.isBlockKeyword = True
        oLine.insideBlock = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        dVars['iCurrentIndentLevel'] += 1


def classify_begin_keyword(dVars, oLine):
        if re.match('^\s*begin', oLine.lineLower) and not oLine.insideFunction and \
           not oLine.insideProcess and not oLine.insideGenerate and \
           not oLine.insideProcedure:
            oLine.isBlockBegin = True
            oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1


def classify_end_keyword(dVars, oLine):
    if not oLine.insideProcess and not oLine.insideCase and \
       not oLine.insideComponent and not oLine.insideGenerate and \
       not oLine.insideFunction and not oLine.insideForLoop and \
       not oLine.insideWhileLoop and not oLine.insideTypeRecord and \
       not oLine.insideProcedure:
        if re.match('^\s*end', oLine.lineLower):
            oLine.isEndBlock = True
            oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
            dVars['iCurrentIndentLevel'] -= 1




from vsg import rule
from vsg import fix

import re


class rule_009(rule.rule):
    '''
    Package rule 009 checks for a single space between the "end" and "package" keywords and component name.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'package', '009')
        self.solution = 'Single space between "end" and "package" keywords and component name.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isPackageEnd:
            check_spaces_between_end_and_package_and_name(self, oLine, iLineNumber)
            check_spaces_between_end_and_package(self, oLine, iLineNumber)
            check_spaces_between_end_and_name(self, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], 'end')
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], 'package')


def check_spaces_between_end_and_package_and_name(self, oLine, iLineNumber):
    if re.match('^\s*end\s+package\s+\w', oLine.lineLower):
        if not re.match('^\s*end\spackage\s\w', oLine.lineLower):
            self.add_violation(iLineNumber)


def check_spaces_between_end_and_package(self, oLine, iLineNumber):
    if re.match('^\s*end\s+package', oLine.lineLower):
        if not re.match('^\s*end\spackage', oLine.lineLower):
            self.add_violation(iLineNumber)


def check_spaces_between_end_and_name(self, oLine, iLineNumber):
    if re.match('^\s*end\s+\w', oLine.lineLower):
        if not re.match('^\s*end\s\w', oLine.lineLower):
            self.add_violation(iLineNumber)



'''
This module contains functions for rules to perform their checks.
'''


def multiline_alignment_with_parenthesis(self, iColumn, oLine, iLineNumber, dParenthesis):
    '''
    Checks the alignment of multiline statements taking parenthesis into account.

    Parameters:

      self: (rule object)

      iColumn: (integer)

      oLine: (line object)

      iLineNumber: (integer)

      dParenthesis: (dictionary)
    '''
    for iIndex in range(dParenthesis[iLineNumber]['begin'], iLineNumber):
        iParenthesisColumn = _find_right_most_open_parenthesis(dParenthesis, iLineNumber)
        if iParenthesisColumn is None:
            if not dParenthesis[iLineNumber]['character'] == iColumn:
                self.add_violation(iLineNumber)
                self.dFix['violations'][iLineNumber] = {}
                self.dFix['violations'][iLineNumber]['column'] = iColumn
        else:
            if not dParenthesis[iLineNumber]['character'] == iParenthesisColumn + 1:
                self.add_violation(iLineNumber)
                self.dFix['violations'][iLineNumber] = {}
                self.dFix['violations'][iLineNumber]['column'] = iParenthesisColumn + 1
                offset = iParenthesisColumn + 1 - dParenthesis[iLineNumber]['character']
                for iIndex in range(len(dParenthesis[iLineNumber]['open'])):
                    dParenthesis[iLineNumber]['open'][iIndex] += offset


def _find_right_most_open_parenthesis(dParenthesis, iLineNumber):
    lOpenParenthesis = []
    for iIndex in range(dParenthesis[iLineNumber]['begin'], iLineNumber):
        for iOpenParenthesis in dParenthesis[iIndex]['open']:
            lOpenParenthesis.insert(0, iOpenParenthesis)
        for iCloseParenthesis in dParenthesis[iIndex]['closed']:
            lOpenParenthesis.pop(0)

    if not len(lOpenParenthesis) == 0:
        return lOpenParenthesis[0]
    return None




from vsg.rules import multiple_spaces_after_rule


class rule_003(multiple_spaces_after_rule):
    '''
    Type rule 003 checks the spaces after the "type" keyword.
    '''

    def __init__(self):
        multiple_spaces_after_rule.__init__(self, 'type', '003', 'isTypeKeyword', 'type')




from vsg.rules import remove_blank_lines_above_rule


class rule_007(remove_blank_lines_above_rule):
    '''
    If rule 007 checks for an empty line before the "elsif" keyword.
    '''

    def __init__(self):
        remove_blank_lines_above_rule.__init__(self, 'if', '007', 'isElseIfKeyword', 'isEndCaseKeyword')
        self.solution = 'Remove blank line(s) before the "elsif" keyword.'




from vsg.rules import line_above_rule


class rule_016(line_above_rule):
    '''
    Architecture rule 016 checks for a blank line above the "begin" keyword.
    '''

    def __init__(self):
        line_above_rule.__init__(self, 'architecture', '016', 'isArchitectureBegin')




from vsg.rules import case_rule
from vsg import utils


class rule_004(case_rule):
    '''
    Type rule 004 checks the type name has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'type', '004', 'isTypeKeyword')
        self.solution = 'Change type identifier name to '

    def _extract(self, oLine):
        return utils.extract_type_identifier(oLine)




from vsg import rule
from vsg import utils


class rule_010(rule.rule):
    '''
    General rule 010 checks capitalization consistency of function names.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'function', '010')
        self.fixable = True
        self.solution = 'Inconsistent capitalization of word'
        self.phase = 6
        self.lLowerCaseWords = []
        self.lCaseWords = []
        self.dDatabase = create_database()

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isFunctionKeyword:
            self.dDatabase['function'].append(oLine.line.split()[1])  # TODO: Think if this can be solved in better way.
        if oLine.insideArchitecture:
            if oLine.insideProcess or oLine.insideConcurrent:
                lWords = extract_word_list(oLine)
                check_violations(self, lWords, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            for sWord in self.dFix['violations'][iLineNumber]:
                sReplacementWord = get_replacement_word(self, sWord)
                utils.change_word(oFile.lines[iLineNumber], sWord, sReplacementWord, 20)

    def _get_solution(self, iLineNumber):
        if len(self.dFix['violations'][iLineNumber]) > 1:
            sSolution = self.solution + 's: ' + ', '.join(self.dFix['violations'][iLineNumber])
        else:
            sSolution = self.solution + ': ' + ', '.join(self.dFix['violations'][iLineNumber])
        return sSolution


def create_database():

    dDatabase = {}
    dDatabase['function'] = []

    return dDatabase


def extract_word_list(oLine):

    sLine = oLine.line

    lWords = utils.extract_non_keywords(sLine)
    return lWords


def check_violations(self, lWords, iLineNumber):
    for sWord in lWords:
        if sWord.lower() in map(str.lower, self.dDatabase['function']):
            if sWord not in self.dDatabase['function']:
                self.add_violation(iLineNumber)
                try:
                    self.dFix['violations'][iLineNumber].append(sWord)
                except KeyError:
                    self.dFix['violations'][iLineNumber] = []
                    self.dFix['violations'][iLineNumber].append(sWord)


def get_replacement_word(self, sWord):
    for sNewWord in self.dDatabase['function']:
        if sNewWord.lower() == sWord.lower():
            return sNewWord




from vsg.rules import indent_rule


class rule_006(indent_rule):
    '''
    Generate rule 006 checks the indent of the generate declaration.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'generate', '006', 'isGenerateBegin')




from vsg.rules import case_rule
from vsg import utils


class rule_019(case_rule):
    '''
    Process rule 019 checks the "end process" label has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'process', '019', 'isEndProcess')
        self.case = 'upper'
        self.solution = 'Change end process label to '

    def _extract(self, oLine):
        return utils.extract_end_label(oLine)




from vsg.rules import indent_rule


class rule_011(indent_rule):
    '''
    Function rule 011 checks the indent of function parameters when they are on multiple lines.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'function', '011', 'isFunctionParameterEnd', 'isFunctionParameter')




from vsg import rule
from vsg import fix

import re


class rule_030(rule.rule):
    '''
    Instantiation rule 030 checks for a single space after the => operator in generic maps.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '030'
        self.solution = 'Only a single space after => operator.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isInstantiationGenericAssignment and not oLine.isInstantiationGenericKeyword:
            if not re.match('^.*=>\s\S+', oLine.line):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], '=>')




from vsg import rule

import re


class rule_007(rule.rule):
    '''
    Package rule 007 checks for the "package" keyword on the end package declaration.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'package'
        self.identifier = '007'
        self.solution = 'End of package follows this format: end package <package name>.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isPackageEnd and not re.match('^\s*end\s+package', oLine.lineLower):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            iIndex = oLine.lineLower.find('end') + len('end')
            oLine.update_line(oLine.line[:iIndex] + ' package' + oLine.line[iIndex:])




from vsg import rule
from vsg import utils

import re


class rule_005(rule.rule):
    '''
    Entity rule 005 checks the "is" keyword is on the same line as the
    entity keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'entity'
        self.identifier = '005'
        self.solution = 'Add "is" keyword to same line as "entity" keyword.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isEntityDeclaration and not re.match('^.*\s\s*is', oLine.lineLower):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub(r'^(\s*entity\s+\w+)', r'\1 is', oLine.line, re.IGNORECASE))
            utils.search_for_and_remove_keyword(oFile, iLineNumber, 'is')




from vsg.rules import case_rule
from vsg import utils


class rule_028(case_rule):
    '''
    If rule 028 checks the **end if** keywords have proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'if', '028', 'isEndIfKeyword')
        self.solution = 'Change "end if" keywords to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['end', 'if'])




from vsg.rules import indent_rule


class rule_009(indent_rule):
    '''Component rule 009 checks for spaces before the "end" keyword.'''

    def __init__(self):
        indent_rule.__init__(self, 'component', '009', 'isComponentEnd')
        self.solution = 'Ensure proper indentation.'




import re

from vsg import rule
from vsg import fix
from vsg import check


class rule_011(rule.rule):
    '''
    Architecture rule 011 checks the architecture name case on the closing "end architecture" line.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'architecture', '011')
        self.solution = 'Uppercase architecture name.'
        self.case = 'upper'
        self.phase = 6
        self.configuration.append('case')

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isEndArchitecture and re.match('^\s*end\s+architecture\s+\w+', oLine.line, re.IGNORECASE):
            lLine = oLine.line.split()
            if self.case == 'upper':
                check.is_uppercase(self, lLine[2], iLineNumber)
            else:
                check.is_lowercase(self, lLine[2], iLineNumber)
            self.dFix[iLineNumber] = 2
        elif oLine.isEndArchitecture and re.match('^\s*end\s+\w+', oLine.line, re.IGNORECASE):
            lLine = oLine.line.split()
            self.dFix[iLineNumber] = 2
            if not lLine[1].lower().startswith('architecture'):
                self.dFix[iLineNumber] = 1
                if self.case == 'upper':
                    check.is_uppercase(self, lLine[1], iLineNumber)
                else:
                    check.is_lowercase(self, lLine[1], iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            iIndex = self.dFix[iLineNumber]
            if self.case == 'upper':
                fix.upper_case(oFile.lines[iLineNumber], oFile.lines[iLineNumber].line.split()[iIndex])
            else:
                fix.lower_case(oFile.lines[iLineNumber], oFile.lines[iLineNumber].line.split()[iIndex])

    def _get_solution(self, iLineNumber):
        if self.case == 'upper':
            return 'Uppercase architecture name.'
        else:
            return 'Lowercase architecture name.'




from vsg.rules import indent_rule


class rule_013(indent_rule):
    '''
    If rule 013 checks the indent of the "else" keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'if', '013', 'isElseKeyword')




from vsg.rules import indent_rule


class rule_007(indent_rule):
    '''
    Generate rule 007 checks the indent of the generate declaration.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'generate', '007', 'isGenerateEnd')



import re


def variable_assignment(dVars, oLine):

    if not (oLine.insideProcess or oLine.insideFunction or oLine.insideProcedure):
        return

    if re.match('^\s*.*\s*:=', oLine.lineNoComment) and \
       not oLine.isComment and not oLine.insideIf and \
       not oLine.isElseKeyword and not oLine.isVariable and not oLine.isSignal \
       and not oLine.isProcedureParameter and not oLine.isFunctionParameter:
        oLine.isVariableAssignment = True
        oLine.insideVariableAssignment = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        oLine.variableAssignmentAlignmentColumn = oLine.lineNoComment.find(':=')
    if oLine.insideVariableAssignment:
        if ';' in oLine.lineNoComment:
            oLine.isVariableAssignmentEnd = True




from vsg import rule
from vsg import fix
from vsg import check

import re


class rule_009(rule.rule):
    '''
    Port rule 009 checks for a single space after "inout" keyword in a port declaration for "inout" ports.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'port'
        self.identifier = '009'
        self.solution = 'Change the number of spaces after the "inout" keyword to one space.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isPortDeclaration and re.match('^\s*\S+\s*:\s*inout', oLine.lineLower):
            check.is_single_space_after(self, 'inout', oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            fix.enforce_one_space_after_word(self, oLine, 'inout')




from vsg.rules import single_space_after_rule


class rule_002(single_space_after_rule):
    '''
    Case rule 002 checks for a single space after the "case" keyword.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'case', '002', 'isCaseKeyword', 'case')
        self.solution = 'Ensure a single space exists after the "case" keyword.'




from vsg.rules import prefix_rule
from vsg import utils


class rule_015(prefix_rule):
    '''
    Type rule 015 checks for prefixes in user defined type identifiers.
    '''

    def __init__(self):
        prefix_rule.__init__(self, 'type', '015', 'isTypeKeyword')
        self.prefixes = ['t_']
        self.solution = 'Type'

    def _extract(self, oLine):
        return utils.extract_type_identifier(oLine)




from vsg.rules import single_space_after_rule


class rule_003(single_space_after_rule):
    '''
    Generic rule 003 checks spacing between "generic" keyword and the open parenthesis.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'generic', '003', 'isGenericKeyword', 'generic')
        self.solution = 'Change spacing between "generic" and "(" to one space.'




from vsg.rules.architecture import architecture_case_rule


class rule_014(architecture_case_rule):
    '''
    Architecture rule 014 checks the entity name is upper case in the architecture declaration.
    '''

    def __init__(self):
        architecture_case_rule.__init__(self, 'architecture', '014', 3)
        self.solution = 'entity name.'




from vsg.rules import case_rule
from vsg import utils


class rule_004(case_rule):
    '''
    Signal rule 004 checks the signal name has proper case.
    '''
    def __init__(self, name='signal', identifier='004'):
        case_rule.__init__(self, 'signal', '004', 'isSignal')
        self.solution = 'Change signal identifiers name to '

    def _extract(self, oLine):
        return utils.extract_class_identifier_list(oLine)




from vsg import rule

import re


class remove_spaces_before_character_rule(rule.rule):
    '''
    This class removes spaces before a given character.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    sCharacter : string
       The character to start removing spaces before.
    '''

    def __init__(self, name=None, identifier=None, sCharacter=None):
        rule.rule.__init__(self, name, identifier)
        self.phase = 2
        self.sCharacter = sCharacter
        self.solution = None

    def _analyze(self, oFile, oLine, iLineNumber):
        if ' ' + self.sCharacter in oLine.lineNoComment:
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub(r'(\s+' + self.sCharacter + ')', self.sCharacter, oLine.line))




from vsg.rules import case_rule
from vsg import utils


class rule_019(case_rule):
    '''
    Entity rule 019 checks the of keyword has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'architecture', '019', 'isArchitectureKeyword')
        self.solution = 'Change "of" keyword to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['of'])




from vsg.rules import case_rule
from vsg import utils


class rule_017(case_rule):
    '''
    Generic rule 017 checks the generic type has proper case if it is a VHDL keyword.
    '''

    def __init__(self):
        case_rule.__init__(self, 'generic', '017', 'isGenericDeclaration')
        self.solution = 'Change generic name to '

    def _extract(self, oLine):
        return utils.extract_type_name_vhdl_only(oLine)




import re

from vsg import rule
from vsg import utils


class rule_002(rule.rule):
    '''
    Checks the case of the **to** keyword in range definitions.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'range', '002')
        self.phase = 6
        self.solution = 'Change to keyword to '
        self.case = 'lower'
        self.configuration.append('case')

    def _analyze(self, oFile, oLine, iLineNumber):
        if re.match('^.*\sto[\s|$]', oLine.lineNoComment, re.IGNORECASE):
            if self.case == 'lower':
                if not re.match('^.*\sto[\s|$]', oLine.lineNoComment):
                    self.add_violation(iLineNumber)
                    _save_string_index_of_to(self, oLine, iLineNumber)
            else:
                if not re.match('^.*\sTO[\s|$]', oLine.lineNoComment):
                    self.add_violation(iLineNumber)
                    _save_string_index_of_to(self, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            iIndex = self.dFix['violations'][iLineNumber]
            if self.case == 'lower':
                utils.replace_word_by_index(oLine, iIndex, ' to')
            else:
                utils.replace_word_by_index(oLine, iIndex, ' TO')

    def _get_solution(self, iLineNumber):
        return self.solution + self.case + 'case.'


def _save_string_index_of_to(self, oLine, iLineNumber):
    sLine = oLine.lineNoComment.lower()
    iIndex = sLine.find(' to')
    self.dFix['violations'][iLineNumber] = iIndex




from vsg.rules import case_rule
from vsg import utils


class rule_011(case_rule):
    '''
    Instantiation rule 011 checks the port name has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'instantiation', '011', 'isInstantiationPortAssignment')
        self.case = 'upper'
        self.solution = 'Change port name to '

    def _extract(self, oLine):
        return utils.extract_port_names_from_port_map(oLine)




from vsg import rule
from vsg import check
from vsg import fix


class single_space_after_rule(rule.rule):
    '''
    Checks for and fixes none or multiple spaces after a word.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    sTrigger : string
       The line attribute the rule applies to.

    sWord : string
       The word to check for a single space after.

    Attributes
    ----------

    self.phase : integer = 2
       Sets the phase the rule will run in.

    self.solution : string = None
       Instructions on how to fix the violation.
    '''

    def __init__(self, name=None, identifier=None, sTrigger=None, sWord=None):
        rule.rule.__init__(self, name=name, identifier=identifier)
        self.sTrigger = sTrigger
        self.sWord = sWord
        self.phase = 2
        self.solution = None
        self.spaces = 1
        self.configuration.append('spaces')

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.__dict__[self.sTrigger]:
            check.is_single_space_after(self, self.sWord, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            fix.enforce_one_space_after_word(self, oLine, self.sWord)

    def _get_solution(self, iLineNumber):
        return 'Ensure there are only ' + str(self.spaces) + ' space(s) after the "' + self.name + '" keyword.'




from vsg import rule
from vsg import check
from vsg import fix
from vsg import utils


class rule_003(rule.rule):
    '''
    Concurrent rule 003 checks the alignment of multiline concurrent assignments.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'concurrent'
        self.identifier = '003'
        self.solution = 'Align first character in row to the column of text one space after the <= or to the open parenthesis in the line above.'
        self.phase = 6

    def _pre_analyze(self):
        self.iAlignmentColumn = 0
        self.dParenthesis = {}

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.insideConcurrent:
            self.dParenthesis[iLineNumber] = {}
            self.dParenthesis[iLineNumber]['open'] = extract_open_parenthesis_locations(oLine.lineNoComment)
            self.dParenthesis[iLineNumber]['closed'] = extract_closed_parenthesis_locations(oLine.lineNoComment)
            self.dParenthesis[iLineNumber]['character'] = utils.begin_of_line_index(oLine)
            self.dParenthesis[iLineNumber]['offset'] = 0
            if oLine.isConcurrentBegin:
                self.dParenthesis[iLineNumber]['begin'] = iLineNumber
                self.dParenthesis[iLineNumber]['align'] = oLine.line.find('<') + 3
                if not oLine.isEndConcurrent:
                    self.iAlignmentColumn = oLine.line.find('<') + 3
            else:
                self.dParenthesis[iLineNumber]['begin'] = self.dParenthesis[iLineNumber - 1]['begin']
                self.dParenthesis[iLineNumber]['align'] = self.dParenthesis[iLineNumber - 1]['align']
                check.multiline_alignment_with_parenthesis(self, self.iAlignmentColumn, oLine, iLineNumber, self.dParenthesis)

            clear_parenthesis_dictionary(self, oLine)

    def _fix_violations(self, oFile):
        for iLineNumber in self.dFix['violations']:
            fix.multiline_alignment(self, oFile, iLineNumber)


def clear_parenthesis_dictionary(self, oLine):
    if oLine.isEndConcurrent:
        self.dParenthesis = {}


def extract_open_parenthesis_locations(sLine):
    lReturn = []
    iIndex = 0
    while True:
        iIndex = sLine.find('(', iIndex)
        if iIndex == -1:
            break
        lReturn.append(iIndex)
        iIndex = iIndex + 1
    return lReturn


def extract_closed_parenthesis_locations(sLine):
    lReturn = []
    iIndex = 0
    while True:
        iIndex = sLine.find(')', iIndex)
        if iIndex == -1:
            break
        lReturn.append(iIndex)
        iIndex = iIndex + 1
    return lReturn



'''
This module contains functions for rules to perform their checks.
'''

import re


def are_there_spaces_after(self, sString, oLine, iLineNumber, iSpaces):
    '''
    Checks if a space is after the string given.
    The string is considered a whole word.
    Allowances are made for end of line and semicolons.

    Parameters:

      self: (rule object)

      sString: (string)

      oLine: (line object)

      iLineNumber: (integer)

      iSpaces: (integer)
    '''
    sSpaces = ' ' * iSpaces
    if not sString.lower() in oLine.lineLower:
        return
    if re.match('^.*' + sString + ';', oLine.lineLower):
        return
    if re.match('^.*' + sString + '$', oLine.lineLower):
        return
    if re.match('^.*' + sString + sSpaces + '\S', oLine.lineLower):
        return
    if not re.match('^.*\s+' + sString + sSpaces + '\S', oLine.lineLower) and \
       not re.match('^\s*' + sString + sSpaces + '\S', oLine.lineLower) and \
       not re.match('^.*\S\s' + sString + '\'', oLine.lineLower):
        self.add_violation(iLineNumber)



import re


def after(dVars, oLine):

    if oLine.isEndConcurrent or oLine.isSequentialEnd or oLine.insideSequential:
        if re.match('^.*\s+after\s', oLine.lineNoComment, flags=re.IGNORECASE):
            oLine.hasAfterKeyword = True
        if re.match('^.*after$', oLine.lineNoComment, flags=re.IGNORECASE):
            oLine.hasAfterKeyword = True




from vsg import rule

import re


class rule_015(rule.rule):
    '''
    Entity rule 015 checks the "end" keyword, "entity" keyword, and entity name are on the same line.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'entity'
        self.identifier = '015'
        self.solution = 'The "end" keyword, "entity" keyword and entity name need to be on the same line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isEndEntityDeclaration:
            if not re.match('^\s*end\s+entity', oLine.line, re.IGNORECASE):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.line = re.sub(r'^(\s*end)', r'\1 entity', oLine.line, flags=re.IGNORECASE)
            oLine.lineLower = oLine.line.lower()




from vsg import rule
from vsg import check
from vsg import fix


class rule_001(rule.rule):
    '''Case rule 001 checks for the proper indentation at the beginning of the line.'''

    def __init__(self):
        rule.rule.__init__(self, 'case', '001')
        self.solution = 'Ensure proper indentation.'
        self.phase = 4

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isCaseKeyword or oLine.isCaseWhenKeyword or oLine.isEndCaseKeyword:
            check.indent(self, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.indent(self, oFile.lines[iLineNumber])



from vsg.rules import indent_rule


class rule_001(indent_rule):
    '''
    Checks the indent level of the "while" keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'while_loop', '001', 'isWhileLoopKeyword')




from vsg.rules import case_rule
from vsg import utils


class rule_004(case_rule):
    '''
    Variable rule 004 checks the variable identifiers have proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'variable', '004', 'isVariable')
        self.solution = 'Change variable identifiers name to '

    def _extract(self, oLine):
        return utils.extract_class_identifier_list(oLine)




from vsg.rules import case_rule
from vsg import utils


class rule_017(case_rule):
    '''
    Process rule 017 checks the process label has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'process', '017', 'isProcessLabel')
        self.case = 'upper'
        self.solution = 'Change label name to '

    def _extract(self, oLine):
        return utils.extract_label(oLine)




from vsg import rule
from vsg import check
from vsg import fix
from vsg import line


class keyword_alignment_rule(rule.rule):
    '''
    Checks for keyword alignment in consecutive concurrent statements.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    Attributes
    ----------

    sKeyword : string
       The keyword to align.

    sStartGroupTrigger: string
       The line attribute which marks the start of a group to align.

    sEndGroupTrigger: string
       The line attribute which ends the group to align.

    sLineTrigger : string
       Stores lines in a group which match this attribute.
    '''

    def __init__(self, name=None, identifier=None):
        rule.rule.__init__(self, name, identifier)
        self.phase = 5
        # The following is filled out by the user
        self.sKeyword = None
        self.sStartGroupTrigger = None
        self.sEndGroupTrigger = None
        self.sLineTrigger = None

    def _pre_analyze(self):
        self.lGroup = []
        self.fGroupFound = False
        self.iStartGroupIndex = None

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.__dict__[self.sStartGroupTrigger] and not self.fGroupFound:
            self.fGroupFound = True
            self.iStartGroupIndex = iLineNumber
        if oLine.__dict__[self.sEndGroupTrigger] and self.fGroupFound:
            self.lGroup.append(oLine)
            self.fGroupFound = False
            check.keyword_alignment(self, self.iStartGroupIndex, self.sKeyword, self.lGroup)
            self.lGroup = []
            self.iStartGroupIndex = None
        if self.fGroupFound:
            if isinstance(self.sLineTrigger, list):
                for sTrigger in self.sLineTrigger:
                    if oLine.__dict__[sTrigger]:
                        self.lGroup.append(oLine)
            elif oLine.__dict__[self.sLineTrigger]:
                self.lGroup.append(oLine)
            else:
                self.lGroup.append(line.line('Removed line'))

    def _fix_violations(self, oFile):
        fix.keyword_alignment(self, oFile)



import re


def generate(dVars, oLine, oLinePrevious):

    classify_case_generate(dVars, oLine) 
    classify_for_generate(dVars, oLine) 
    classify_if_generate(dVars, oLine) 
    classify_multi_line_generate_statement(dVars, oLine, oLinePrevious)
    if oLine.insideGenerate:
        if re.match('^\s*\w+\s*:\s*if[\s|(].*[\s|)]generate', oLine.lineLower) or re.match('^\s*\w+\s*:\s*for\s.*\sgenerate', oLine.lineLower) or re.match('^\s*\w+\s*:\s*case\s.*\sgenerate', oLine.lineLower):
            oLine.isGenerateKeyword = True
            oLine.isGenerateLabel = True
            oLine.indentLevel = dVars['iCurrentIndentLevel']
            dVars['iCurrentIndentLevel'] += 1
        if re.match('^\s*if[\s|(].*[\s|)]generate', oLine.lineLower) or re.match('^\s*for\s.*\sgenerate', oLine.lineLower) or re.match('^\s*case\s.*\sgenerate', oLine.lineLower):
            oLine.isGenerateKeyword = True
            oLine.indentLevel = dVars['iCurrentIndentLevel']
            dVars['iCurrentIndentLevel'] += 1
        if re.match('^\s*begin', oLine.lineLower) and not oLine.insideProcess:
            oLine.isGenerateBegin = True
            oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
        classify_end_generate(dVars, oLine)

        classify_for_generate_case_when(dVars, oLine)


def classify_end_generate(dVars, oLine):
    if re.match('^\s*end\s+generate', oLine.lineLower):
        oLine.isGenerateEnd = True
        dVars['iCurrentIndentLevel'] -= 1
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        dVars['iGenerateLevel'] -= 1
        if re.match('^\s*end\s+generate\s+\S+\s*;', oLine.lineLower):
            oLine.isGenerateEndLabel = True
        if oLine.insideGenerateCase:
            oLine.indentLevel -= 1
            dVars['iCurrentIndentLevel'] -= 1


def classify_for_generate(dVars, oLine):
    if re.match('^\s*\w+\s*:\s*for\s.*\sgenerate', oLine.lineLower):
        oLine.insideGenerate = True
        dVars['iGenerateLevel'] += 1


def classify_if_generate(dVars, oLine):
    if re.match('^\s*\w+\s*:\s*if', oLine.lineLower):
        oLine.insideGenerate = True
        dVars['iGenerateLevel'] += 1


def classify_case_generate(dVars, oLine):
    if re.match('^\s*\w+\s*:\s*case\s.*\sgenerate', oLine.lineLower):
        oLine.insideGenerate = True
        oLine.insideGenerateCase = True
        dVars['iGenerateLevel'] += 1


def classify_for_generate_case_when(dVars, oLine):
    if oLine.insideGenerateCase:
        if re.match('^\s*when\s+\w+\s+=>', oLine.lineLower):
            oLine.isGenerateCaseWhen = True
            if not dVars['bFirstWhenSeen']:
                oLine.indentLevel = dVars['iCurrentIndentLevel']
                dVars['bFirstWhenSeen'] = True
                dVars['iCurrentIndentLevel'] += 1
            else:
                oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1

def classify_multi_line_generate_statement(dVars, oLine, oLinePrevious):
    if re.match('^\s*\w+\s*:\s*--', oLinePrevious.line) or re.match('^\s*\w+\s*:\s*$', oLinePrevious.line):
        if re.match('^\s*if\s+.*\sgenerate', oLine.lineLower) or re.match('^\s*for\s.*\sgenerate', oLine.lineLower) or re.match('^\s*case\s+.*\sgenerate', oLine.lineLower):
            oLinePrevious.insideGenerate = True
            oLinePrevious.isGenerateLabel = True
            oLine.insideGenerate = True
            dVars['iGenerateLevel'] += 1
            




from vsg import rule
from vsg import line
from vsg import utils

import re


class rule_008(rule.rule):
    '''
    Type rule 008 checks the closing parenthesis of a multi-line type declaration is on it's own line.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'type', '008')
        self.solution = 'Move the closing parenthesis to it\'s own line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isTypeEnumeratedEnd and not oLine.isTypeEnumeratedKeyword:
            if not re.match('^\s*\)\s*;', oLine.lineLower):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            oFile.lines[iLineNumber].line = utils.remove_closing_parenthesis_and_semicolon(oFile.lines[iLineNumber].line)
            oFile.lines[iLineNumber].isTypeEnd = False
            oFile.lines[iLineNumber].isTypeEnumeratedEnd = False
            oFile.lines.insert(iLineNumber + 1, line.line('  );'))
            oFile.lines[iLineNumber + 1].isTypeEnd = True
            oFile.lines[iLineNumber + 1].isTypeEnumeratedEnd = True
            oFile.lines[iLineNumber + 1].insideTypeEnumerated = True
            oFile.lines[iLineNumber + 1].indentLevel = oFile.lines[iLineNumber].indentLevel - 1




from vsg import rule


class rule_014(rule.rule):
    '''
    Package rule 014 checks for the package name exists on the same line as the "end" and "package" keywords.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'package', '014')
        self.solution = 'End of package follows this format: end package <package name>.'
        self.phase = 1
        self.fixable = False  # User must add package name

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isPackageEnd:
            lLine = oLine.lineLower.replace(';', '').split()
            if len(lLine) < 3 and lLine[1] == 'package':
                self.add_violation(iLineNumber)




from vsg.rules import single_space_after_rule


class rule_012(single_space_after_rule):
    '''
    Architecture rule 012 checks for a single space between the "end" and "architecture" keywords.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'architecture', '012', 'isEndArchitecture', 'end')
        self.solution = 'Single space between "end" and "architecture" keywords.'




from vsg import rule

import re


class rule_020(rule.rule):
    '''
    Case rule 020 checks for labels after the "end case" keywords.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'case', '020')
        self.phase = 1
        self.solution = 'Remove label after the "end case" keywords'

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.hasEndCaseLabel:
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub('^(\s*end\s+case)(\s*\w+\s*)(;\s*$)', r'\1\3', oLine.line, 1, flags=re.IGNORECASE))
            oLine.hasEndCaseLabel = False




from vsg.rules import indent_rule


class rule_014(indent_rule):
    '''
    If rule 014 checks the indent of the "end if" keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'if', '014', 'isEndIfKeyword')




from vsg.rules import search_for_and_replace_keyword_rule


class rule_018(search_for_and_replace_keyword_rule):
    '''
    Generic rule 018 checks the **generic** keyword is on the same line as the (.
    '''

    def __init__(self):
        search_for_and_replace_keyword_rule.__init__(self, 'generic', '018')
        self.solution = 'Move the ( to the same line as the "generic" keyword.'
        self.sKeyword = '('
        self.sTrigger = 'isGenericKeyword'
        self.sKeyword2 = 'generic'




from vsg import line
from vsg.vhdlFile import update
from vsg.vhdlFile import classify


class vhdlFile():
    '''
    Holds contents of a VHDL file.
    When a vhdlFile object is created, the contents of the file must be passed to it.
    A line object is created for each line read in.
    Then the line object attributes are updated.

    Parameters:

       filecontent: (list)

    Returns:

       fileobject
    '''
    def __init__(self, filecontent):
        self.filecontent = filecontent
        self.lines = [line.line('')]
        self.hasArchitecture = False
        self.hasEntity = False
        self._processFile()
        self.filename = None

    def _processFile(self):

        dVars = {}
        dVars['fFoundProcessBegin'] = False
        dVars['SensitivityListFound'] = False
        dVars['fProcedureParameterEndDetected'] = False
        dVars['fProcedureIsDetected'] = False
        dVars['fProcedureBeginDetected'] = False
        dVars['fFunctionParameterEndDetected'] = False
        dVars['fFunctionIsDetected'] = False
        dVars['fFunctionBeginDetected'] = False
        dVars['iOpenParenthesis'] = 0
        dVars['iCloseParenthesis'] = 0
        dVars['iCurrentIndentLevel'] = 0
        dVars['iGenerateLevel'] = 0
        dVars['iIfLevel'] = 0
        dVars['fConstantArray'] = False
        dVars['iForLoopLevel'] = 0
        dVars['bFirstWhenSeen'] = False

        oLinePrevious = line.blank_line()

        for sLine in self.filecontent:
            oLine = line.line(sLine.replace('\t', '  ').rstrip())
#            preIndent = str(dVars['iCurrentIndentLevel'])
            update.inside_attributes(dVars, self.lines[-1], oLine)

            classify.blank(oLine)
            classify.comment(dVars, oLine)
            classify.library(oLine)
            classify.entity(self, dVars, oLine)
            classify.assert_statement(dVars, oLine)

            classify.code_tags(dVars, oLine, oLinePrevious)

            classify.port(dVars, oLine)
            classify.generic(dVars, oLine)

            classify.concurrent(dVars, oLine)
            classify.architecture(self, dVars, oLine)
            classify.package_body(dVars, oLine)
            classify.block(self, dVars, oLine)
            classify.package(dVars, oLine)
            classify.component(dVars, oLine)
            classify.signal(dVars, oLine)
            classify.constant(dVars, oLine, oLinePrevious)
            classify.variable(dVars, oLine)
            classify.procedure(dVars, oLine, oLinePrevious)
            classify.process(dVars, oLine, self.lines)
            classify.generate(dVars, oLine, oLinePrevious)
            classify.attribute(dVars, oLine)
            classify.file_statement(dVars, oLine)

            classify.when(dVars, oLine, oLinePrevious)

            classify.with_statement(dVars, oLine)
            classify.for_loop(dVars, oLine)
            classify.while_loop(dVars, oLine)

            classify.if_statement(dVars, oLine)

            classify.case(self, dVars, oLine)
            classify.function(dVars, oLine)
            classify.type_definition(dVars, oLine)
            classify.subtype(dVars, oLine)

            classify.sequential(dVars, oLine)
            classify.variable_assignment(dVars, oLine)
            classify.wait(dVars, oLine)
            classify.after(dVars, oLine)

            # Check instantiation statements
            if oLine.insideArchitecture and not oLine.insideProcess and \
               not oLine.isConcurrentBegin and \
               not oLine.insideComponent and \
               not oLine.isGenerateKeyword and \
               not oLine.insideFunction and \
               not oLine.insideProcedure:
                classify.instantiation(dVars, oLine)

            # Add line to file
            self.lines.append(oLine)
            oLinePrevious = oLine
#            print('[' + preIndent + '][' + oLine.line + '][' + str(oLine.indentLevel) + ']')




from vsg.rules import case_rule
from vsg import utils


class rule_027(case_rule):
    '''
    Instantiation rule 027 checks the **entity** keyword has proper case in direct instantiations.
    '''

    def __init__(self):
        case_rule.__init__(self, 'instantiation', '027', 'isDirectInstantiationDeclaration')
        self.solution = 'Change "entity" keyword to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['entity'])




from vsg import rule

import copy


class rule_005(rule.rule):
    '''
    Instantiation rule 005 checks the instantiation declaration and
    "port map" keywords are not on the same line.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '005'
        self.solution = 'Place "port map" keywords on the next line by itself'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isInstantiationDeclaration and oLine.isInstantiationPortKeyword:
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            oLine = oFile.lines[iLineNumber]
            iIndex = oLine.lineLower.find(' port ')
            oFile.lines.insert(iLineNumber + 1, copy.deepcopy(oLine))
            oLine.update_line(oLine.line[:iIndex])
            oLine.isInstantiationPortKeyword = False
            oLine.isInstantiationPortEnd = False
            oLine = oFile.lines[iLineNumber + 1]
            oLine.update_line(oLine.line[iIndex:])
            oLine.isInstantiationDeclaration = False
            oLine.indentLevel += 1



from .after import after
from .architecture import architecture
from .assert_statement import assert_statement
from .attribute import attribute
from .blank import blank
from .case import case
from .code_tags import code_tags
from .comment import comment
from .component import component
from .concurrent import concurrent
from .constant import constant
from .entity import entity
from .file_statement import file_statement
from .for_loop import for_loop
from .function import function
from .generate import generate
from .generic import generic
from .if_statement import if_statement
from .instantiation import instantiation
from .library import library
from .package_body import package_body
from .package import package
from .port import port
from .process import process
from .procedure import procedure
from .sequential import sequential
from .signal import signal
from .subtype import subtype
from .type_definition import type_definition
from .variable_assignment import variable_assignment
from .variable import variable
from .when import when
from .while_loop import while_loop
from .with_statement import with_statement
from .block import block
from .wait import wait




from vsg.rules import case_rule
from vsg import check


class rule_008(case_rule):
    '''
    Package rule 008 checks the package name has proper case on the closing "end package" line.
    '''

    def __init__(self):
        case_rule.__init__(self, 'package', '008', 'isPackageEnd')
        self.solution = 'Change package name to '
        self.case = 'upper'

    def _extract(self, oLine):
        if not check.has_package_name(oLine):
            return []

        return [oLine.line.replace(';', '').split()[-1]]




from vsg import rule
from vsg import utils


class rule_015(rule.rule):
    '''
    Signal rule 015 checks for multiple signal names defined in a single signal declaration.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'signal', '015')
        self.solution = 'multiple signals defined in a single declaration'
        self.phase = 1
        self.consecutive = 2
        self.configuration.append('consecutive')

    def _pre_analyze(self):
        self.sFullLine = ''
        self.iFailureLine = None

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isSignal:
            self.sFullLine = ''
            self.iFailureLine = iLineNumber
        if oLine.insideSignal:
            self.sFullLine += oLine.line
        if oLine.isEndSignal:
            if utils.remove_comment(self.sFullLine).count(',') > self.consecutive - 1:
                self.add_violation(self.iFailureLine)
                self.dFix['violations'][self.iFailureLine] = {}
                self.dFix['violations'][self.iFailureLine]['endLine'] = iLineNumber
                self.dFix['violations'][self.iFailureLine]['line'] = self.sFullLine

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            dViolation = self.dFix['violations'][iLineNumber]
            utils.remove_lines(oFile, iLineNumber, dViolation['endLine'])
            iNumLines = utils.remove_comment(dViolation['line']).count(',') + 1
            lSignals = _extract_signals(dViolation['line'])
            sAfterColon = _extract_after_colon(dViolation['line'])
            for i in range(0, iNumLines):
                utils.insert_line(oFile, i + iLineNumber)
                oLine = oFile.lines[i + iLineNumber]
                oLine.isSignal = True
                oLine.insideSignal = True
                oLine.isEndSignal = True
                oLine.isBlank = False
                oLine.update_line('  signal ' + lSignals[i] + ' : ' + sAfterColon)
                utils.update_comment_line_attributes(oLine)


def _extract_signals(sString):
    sString2 = sString.replace(',', ' ')
    sString3 = sString2.split(':')[0]
    lSignals = sString3.split()[1::]
    return lSignals


def _extract_after_colon(sString):
    return sString.split(':')[1].lstrip()



import re


def port(dVars, oLine):
    if not (oLine.insideEntity or oLine.insideComponent):
        return
    classify_port_keyword(dVars, oLine)
    if oLine.isPortKeyword and '(' not in oLine.line:
        return

    if oLine.insidePortMap:
        classify_port_declaration(dVars, oLine)
        classify_port_end_keyword(dVars, oLine)


def classify_port_keyword(dVars, oLine):
    if re.match('^\s*port', oLine.lineLower) and not oLine.insidePortMap and not oLine.insideGenericMap:
        oLine.isPortKeyword = True
        oLine.insidePortMap = True
        if oLine.insideEntity:
            oLine.indentLevel = 1
            dVars['iCurrentIndentLevel'] = 2
        else:
            oLine.indentLevel = 2
            dVars['iCurrentIndentLevel'] = 3


def classify_port_declaration(dVars, oLine):
        if re.match('^\s*\w+.*:', oLine.line) and not oLine.isComment and not oLine.isPortKeyword:
            oLine.isPortDeclaration = True
            if oLine.insideEntity:
                oLine.indentLevel = 2
            else:
                oLine.indentLevel = 3


def classify_port_end_keyword(dVars, oLine):
        dVars['iOpenParenthesis'] += oLine.lineNoComment.count('(')
        dVars['iCloseParenthesis'] += oLine.lineNoComment.count(')')
        if dVars['iOpenParenthesis'] == dVars['iCloseParenthesis']:
            dVars['iOpenParenthesis'] = 0
            dVars['iCloseParenthesis'] = 0
            oLine.isEndPortMap = True
            if oLine.insideEntity:
                oLine.indentLevel = 1
                dVars['iCurrentIndentLevel'] = 1
            else:
                oLine.indentLevel = 2
                dVars['iCurrentIndentLevel'] = 2




from vsg.rules import case_rule
from vsg import utils


class rule_014(case_rule):
    '''
    Component rule 014 checks the is keyword has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'component', '014', 'isComponentEnd')
        self.solution = 'Change "component" keyword to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['component'])




def is_no_blank_line_before(self, oFile, iLineNumber, sUnless=None):
    '''
    Adds a violation if the line before iLineNumber is blank.
    This is typically used to compress lines together.

    If sUnless is given, then a violation will not occur if there is a blank line before the line with the attribute given for sUnless.

    Parameters

      self: (rule object)

      oLine: (line object)

      iLineNumber: (integer)

      sUnless: (string) (line attribute)
    '''
    if sUnless:
        if oFile.lines[iLineNumber - 1].isBlank:
            if not oFile.lines[iLineNumber - 2].__dict__[sUnless]:
                self.add_violation(iLineNumber)
    elif oFile.lines[iLineNumber - 1].isBlank:
        self.add_violation(iLineNumber)




from vsg import rule
from vsg import check
from vsg import fix
from vsg import line


class rule_005(rule.rule):
    '''
    Sequential rule 005 ensures the alignment of the "<=" keyword over
    multiple lines.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'sequential'
        self.identifier = '005'
        self.solution = 'Inconsistent alignment of "<=" in group of lines.'
        self.phase = 5
        self.lGroup = []
        self.fGroupFound = False
        self.iStartGroupIndex = None

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isSequential and not self.fGroupFound:
            self.fGroupFound = True
            self.iStartGroupIndex = iLineNumber
        if not oLine.insideSequential and self.fGroupFound:
            self.fGroupFound = False
            check.keyword_alignment(self, self.iStartGroupIndex, '<=', self.lGroup)
            self.lGroup = []
            self.iStartGroupIndex = None
        if self.fGroupFound:
            if oLine.isComment:
                self.lGroup.append(line.line('Removed line'))
            else:
                self.lGroup.append(oLine)

    def _fix_violations(self, oFile):
        fix.keyword_alignment(self, oFile)




def is_uppercase(self, sString, iLineNumber):
    '''
    Checks if a string is uppercase.

    Parameters:

      self: (rule object)

      sString: (string)

      iLineNumber: (integer)
    '''
    if not sString == sString.upper():
        self.add_violation(iLineNumber)
        return False

    return True




from vsg import rule
from vsg import utils

import re


class rule_011(rule.rule):
    '''Whitespace rule 011 checks for spaces before and after math operators.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'whitespace'
        self.identifier = '011'
        self.phase = 2
        self.solution = 'Add a single space before and/or after math operator.'

    def _analyze(self, oFile, oLine, iLineNumber):
        sLine = utils.remove_comment(oLine.line)
        if sLine[-2:] == '--':
            sLine = sLine[:-2]
        if '-' in sLine:
            lLine = sLine.split()
            for sWord in lLine:
                if '-' in sWord and not sWord == '-':
                    if re.match('^.*\w-', sWord):
                        self.add_violation(iLineNumber)
                    elif not re.match('^.*-[0-9]+\)?$', sWord):
                        self.add_violation(iLineNumber)
        else:
            if re.match('^.*[\w+|\)][+|/|*]', sLine) or re.match('^.*[+|/|*][\w+|\(]', sLine):
                if not re.match('^.*".*/.*"', sLine):
                    self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            iCommentIndex = oLine.line.find('--')
            if iCommentIndex == -1:
                oLine.update_line(re.sub(r'(\w+)([+|\-|/|*])', r'\1 \2', oLine.line))
                oLine.update_line(re.sub(r'\)([+|\-|/|*])', r') \1', oLine.line))
                oLine.update_line(re.sub(r'([+|\-|/|*])(\w+)', r'\1 \2', oLine.line))
                oLine.update_line(re.sub(r'([+|\-|/|*])\(', r'\1 (', oLine.line))
            else:
                sLine = oLine.line[:iCommentIndex]
                sLine = re.sub(r'(\w+)([+|\-|/|*])', r'\1 \2', sLine)
                sLine = re.sub(r'\)([+|\-|/|*])', r') \1', sLine)
                sLine = re.sub(r'([+|\-|/|*])(\w+)', r'\1 \2', sLine)
                sLine = re.sub(r'([+|\-|/|*])\(', r'\1 (', sLine)
                sLine = sLine + oLine.line[iCommentIndex:]
                oLine.update_line(sLine)




from vsg.rules import line_below_rule


class rule_022(line_below_rule):
    '''
    Process rule 022 checks for a blank line below the "begin" keyword.
    '''

    def __init__(self):
        line_below_rule.__init__(self, 'process', '022', 'isProcessBegin')




from vsg.rules import indent_rule


class rule_008(indent_rule):
    '''
    Generic rule 008 checks the indentation of closing parenthesis for generic maps.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'generic', '008', 'isEndGenericMap', 'isGenericDeclaration')




from vsg.rules import indent_rule


class rule_013(indent_rule):
    '''
    Case rule 013 verifies the indent of the "Null" keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'case', '013', 'isCaseNull')




from vsg import rule

import re


class rule_010(rule.rule):
    '''
    Architecture rule 010 checks for "architecture" in the "end architecture statement.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'architecture'
        self.identifier = '010'
        self.solution = 'Add "architecture" keyword after "end" keyword.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isEndArchitecture:
            if not re.match('^\s*end\s+architecture', oLine.line, re.IGNORECASE):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub(r'^(\s*end)', r'\1 architecture', oLine.line, flags=re.IGNORECASE))




from vsg.rules import single_space_after_rule


class rule_007(single_space_after_rule):
    '''
    Process rule 007 checks for a single space between the "end" and "process" keywords.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'process', '007', 'isEndProcess', 'end')
        self.solution = 'Ensure there are only one space between the "end" and "process" keywords.'




from vsg import rule


class rule_007(rule.rule):
    '''
    Signal rule 007 checks for default assignments in signal declarations.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'signal', '007')
        self.solution = 'Remove default assignment.'
        self.phase = 1
        self.fixable = False  # Allow the user to decide if these should be removed

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isSignal and ':=' in oLine.line:
            self.add_violation(iLineNumber)



import re


def procedure(dVars, oLine, oLinePrevious):
    '''
    Classifies procedure statements.

    procedure identifier [ (parameter_interface_list) ] is
      [ subprogram_declarative_part ]
    begin
      { sequential_statement }
    end [procedure] [identifier]

    Sets the following line attributes:

      * insideProcedure
      * insideProcedureDeclarative
      * isProcedureParameter
      * isProcedureBegin
      * isProcedureKeyword
      * isProcedureEnd
      * isProcedureReturn
      * isProcedureIs

    Modifies the following variables:

      * iOpenParenthesis
      * iCurrentIndentLevel
      * fProcedureParameterEndDetected
      * fProcedureIsDetected
    '''
    _classify_procedure_keyword(dVars, oLine)
    if oLine.insideProcedure:
        dVars['iOpenParenthesis'] += oLine.lineNoComment.count('(')
        dVars['iCloseParenthesis'] += oLine.lineNoComment.count(')')
        _classify_procedure_parameters(dVars, oLine)
        _classify_procedure_parameter_end(dVars, oLine, oLinePrevious)
        if oLine.insidePackage:
            _classify_end_procedure_in_package(dVars, oLine)
        else:
            _classify_procedure_declarative_region(dVars, oLine)
            _classify_is_keyword(dVars, oLine)
            _classify_begin_keyword(dVars, oLine)
            _classify_end_procedure_in_package_body(dVars, oLine)


def _classify_procedure_keyword(dVars, oLine):
    if re.match('^\s*procedure\s', oLine.lineLower):
        oLine.isProcedureKeyword = True
        oLine.insideProcedure = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        dVars['iCurrentIndentLevel'] += 1


def _classify_procedure_declarative_region(dVars, oLine):
    if dVars['fProcedureIsDetected'] and not dVars['fProcedureBeginDetected']:
        oLine.insideProcedureDeclarative = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']


def _classify_procedure_parameters(dVars, oLine):
    if re.match('^.*\w+\s*:\s*\w+', oLine.line) and not dVars['fProcedureParameterEndDetected']:
        oLine.isProcedureParameter = True
        if not oLine.isProcedureKeyword:
            oLine.indentLevel = dVars['iCurrentIndentLevel']


def _classify_end_procedure_in_package(dVars, oLine):
    if ';' in oLine.line and dVars['iOpenParenthesis'] == dVars['iCloseParenthesis']:
        dVars['iOpenParenthesis'] = 0
        dVars['iCloseParenthesis'] = 0
        oLine.isProcedureEnd = True
        dVars['iCurrentIndentLevel'] -= 1
        dVars['fProcedureParameterEndDetected'] = False
        dVars['fProcedureIsDetected'] = False
        dVars['fProcedureBeginDetected'] = False


def _classify_procedure_parameter_end(dVars, oLine, oLinePrevious):
    if ')' in oLine.line and dVars['iOpenParenthesis'] == dVars['iCloseParenthesis']:
        dVars['iOpenParenthesis'] = 0
        dVars['iCloseParenthesis'] = 0
        if re.match('^\s*\)\s*is', oLine.line, re.IGNORECASE):
            oLinePrevious.isProcedureParameterEnd = True
            oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
        else:
            oLine.isProcedureParameterEnd = True
        dVars['fProcedureParameterEndDetected'] = True


def _classify_begin_keyword(dVars, oLine):
    if re.match('^\s*begin', oLine.lineLower):
        oLine.isProcedureBegin = True
        oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
        oLine.insideProcedureDeclarative = False
        dVars['fProcedureBeginDetected'] = True


def _classify_is_keyword(dVars, oLine):
    if re.match('^.*[\s|)]is', oLine.lineLower) and dVars['fProcedureParameterEndDetected'] and not dVars['fProcedureIsDetected']:
        oLine.isProcedureIs = True
        dVars['fProcedureIsDetected'] = True
    if re.match('^\s*procedure\s+\w+\s+is', oLine.line, re.IGNORECASE):
        oLine.isProcedureIs = True
        dVars['fProcedureIsDetected'] = True


def _classify_end_procedure_in_package_body(dVars, oLine):
    if re.match('^\s*end', oLine.lineLower) and dVars['iIfLevel'] == 0 and \
       not oLine.isEndCaseKeyword and not oLine.isForLoopEnd and not oLine.isWhileLoopEnd and \
       not oLine.insideForLoop:
        oLine.isProcedureEnd = True
        oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
        dVars['iCurrentIndentLevel'] -= 1
        dVars['iOpenParenthesis'] = 0
        dVars['iCloseParenthesis'] = 0
        dVars['fProcedureParameterEndDetected'] = False
        dVars['fProcedureIsDetected'] = False
        dVars['fProcedureBeginDetected'] = False




from vsg.rules import case_rule
from vsg import utils


class rule_029(case_rule):
    '''
    If rule 029 checks the **then** keyword has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'if', '029', 'isThenKeyword')
        self.solution = 'Change "then" keyword to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['then'])




from vsg.rules import line_above_rule


class rule_003(line_above_rule):
    '''
    Component rule 003 checks for a blank line above the component keyword.
    '''

    def __init__(self):
        line_above_rule.__init__(self)
        self.name = 'component'
        self.identifier = '003'
        self.condition = 'isComponentDeclaration'




from vsg.rules import case_rule
from vsg import utils


class rule_008(case_rule):
    '''
    Instantiation rule 008 checks the instance name has proper case in the instantiation declaration line.
    '''

    def __init__(self):
        case_rule.__init__(self, 'instantiation', '008', 'isInstantiationDeclaration')
        self.case = 'upper'
        self.solution = 'Change instance name to '

    def _extract(self, oLine):
        return utils.extract_label(oLine)




from vsg.rules import indent_rule


class rule_008(indent_rule):
    '''
    Library rule 008 checks indentation of the use keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'library', '008', 'isLibraryUse')




from vsg.rules import indent_rule


class rule_002(indent_rule):
    '''
    Procedure rule 002 checks the indent of the **begin** keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'procedure', '002', 'isProcedureBegin')




from vsg.rules import indent_rule


class rule_002(indent_rule):
    '''
    Port rule 002 checks indentation of the "port" keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'port', '002', 'isPortKeyword')




from vsg.rules import keyword_alignment_rule


class rule_017(keyword_alignment_rule):
    '''
    Component rule 017 ensures the alignment of the : operator for every
    port in the component.
    '''

    def __init__(self):
        keyword_alignment_rule.__init__(self, 'component', '017')
        self.solution = 'Inconsistent alignment of ":" in port declaration of component.'
        self.sKeyword = ':'
        self.sStartGroupTrigger = 'isPortKeyword'
        self.sEndGroupTrigger = 'isEndPortMap'
        self.sLineTrigger = 'isPortDeclaration'




from vsg.rules import indent_rule


class rule_002(indent_rule):
    '''
    Generic rule 002 checks indentation of the "generic" keyword.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'generic', '002', 'isGenericKeyword')




from vsg import rule
from vsg import fix

import re


class rule_002(rule.rule):
    '''
    Architecture rule 002 checks for a single space between "architecture", "of", and "is" keywords.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'architecture'
        self.identifier = '002'
        self.solution = 'Remove extra spaces after architecture keyword.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isArchitectureKeyword and \
           len(oLine.line.split()) > 4 and \
           not re.match('^\s*architecture\s\S+\sof\s\S+\sis', oLine.lineLower):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], 'architecture')
            fix.enforce_one_space_before_word(self, oFile.lines[iLineNumber], 'of')
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], 'of')
            fix.enforce_one_space_before_word(self, oFile.lines[iLineNumber], 'is')




from vsg import rule

import re


class rule_015(rule.rule):
    '''
    Component rule 015 checks for the "component" keyword in the "end component" statement.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'component'
        self.identifier = '015'
        self.solution = 'Add "component" keyword'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isComponentEnd and not re.match('^\s*end\s+component', oLine.line, re.IGNORECASE):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            lLine = oLine.line.split()
            if not lLine[1].lower() == 'component':
                lLine.insert(1, 'component')
                oLine.update_line(' '.join(lLine))




from vsg import rule
from vsg import check
from vsg import fix


class indent_rule(rule.rule):
    '''
    Checks for and fixes indent violations.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    sTrigger : string
       The line attribute the rule applies to.

    sUnless : string
       If set, will not check if attribute matches.
    '''

    def __init__(self, name=None, identifier=None, sTrigger=None, sUnless=None):
        rule.rule.__init__(self, name, identifier)
        self.solution = 'Invalid indentation.'
        self.phase = 4
        self.sTrigger = sTrigger
        self.sUnless = sUnless

    def _analyze(self, oFile, oLine, iLineNumber):
        if self.sUnless:
            if oLine.__dict__[self.sTrigger] and not oLine.__dict__[self.sUnless]:
                check.indent(self, oLine, iLineNumber)
        elif oLine.__dict__[self.sTrigger]:
            check.indent(self, oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.indent(self, oFile.lines[iLineNumber])




import re

from vsg import rule
from vsg import utils


class rule_001(rule.rule):
    '''
    With rule 001 checks the when and else keywords are on the same line
    '''

    def __init__(self):
        rule.rule.__init__(self, 'when', '001')
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.insideWhen:
            if re.match('^\s*else', oLine.line, flags=re.IGNORECASE):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oPreviousLine = oFile.lines[iLineNumber - 1]
            oLine = oFile.lines[iLineNumber]
            iIndex = utils.end_of_line_index(oPreviousLine)
            oPreviousLine.update_line(oPreviousLine.line[:iIndex] + ' else' + oPreviousLine.line[iIndex:])
            utils.clear_keyword_from_line(oLine, 'else')




from vsg import rule
from vsg import check
from vsg import fix

import re


class rule_022(rule.rule):
    '''
    Architecture rule 022 checks for a single space after the "end architecture" keywords and the architecture name.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'architecture', '022')
        self.solution = 'Ensure a single space exists between "architecture" and the architecture name.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isEndArchitecture and re.match('^\s*end\s+architecture\s+\w', oLine.lineLower):
            check.is_single_space_after(self, 'architecture', oLine, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], 'architecture')




from vsg.rules import case_rule
from vsg import utils


class rule_018(case_rule):
    '''
    Port rule 018 checks the port type has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'port', '018', 'isPortDeclaration')
        self.solution = 'Change port type name to '

    def _extract(self, oLine):
        return utils.extract_type_name_from_port_vhdl_only(oLine)




from vsg import rule
from vsg import fix

import re


class rule_032(rule.rule):
    '''
    Instantiation rule 032 checks for a single space after the component keyword if used.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '032'
        self.solution = 'Lowercase the component keyword'
        self.phase = 2
        self.disable = True

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isInstantiationDeclaration and not oLine.isDirectInstantiationDeclaration:
            if re.match('^\s*\w+\s*:\s*component', oLine.line, flags=re.IGNORECASE):
                if not re.match('^\s*\w+\s*:\s*component\s\w', oLine.line, flags=re.IGNORECASE):
                    self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.enforce_one_space_after_word(self, oFile.lines[iLineNumber], 'component')




from vsg import rule

import re


class rule_008(rule.rule):
    '''
    Port rule 008 checks for three spaces after "out" keyword in a port declaration for "out" ports.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'port', '008')
        self.solution = 'Change the number of spaces after the "out" keyword to three spaces.'
        self.phase = 2

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isPortDeclaration and re.match('^\s*\S+\s*:\s*out', oLine.lineLower):
            if not re.match('^\s*\S+\s*:\s*out\s\s\s\S+', oLine.lineLower):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub(r'^(\s*\S+\s*:\s*out)(\s*)', r'\1   ', oLine.line, flags=re.IGNORECASE))




from vsg import rule
from vsg import utils


class rule_013(rule.rule):
    '''Generic rule 013 checks for a generic keyword on the same line as a generic declaration.'''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'generic'
        self.identifier = '013'
        self.solution = 'Move generic declaration to it\'s own line.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isGenericDeclaration and oLine.isGenericKeyword:
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            utils.copy_line(oFile, iLineNumber)
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(extract_generic_keyword(oLine.line))
            oLine.isGenericDeclaration = False
            oLine = oFile.lines[iLineNumber + 1]
            oLine.update_line('    ' + extract_generic_definition(oLine.line))
            oLine.isGenericKeyword = False
            oLine.isGenericDeclaration = True
            oLine.insideGenericMap = True
            oLine.indentLevel = oFile.lines[iLineNumber].indentLevel + 1


def extract_generic_keyword(sString):
    return sString.split('(')[0] + ' ('


def extract_generic_definition(sString):
    lString = sString.split('(')
    return '('.join(lString[1:])




from vsg import rule
from vsg import fix


class rule_015(rule.rule):
    '''
    Process rule 015 checks for a blank line or a comment line above the "process" keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'process', '015')
        self.solution = 'Add a space or a comment above the "process" keyword.'
        self.phase = 3

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isProcessKeyword and \
           not oFile.lines[iLineNumber - 1].isBlank and \
           not oFile.lines[iLineNumber - 1].isComment:
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            fix.insert_blank_line_above(self, oFile, iLineNumber)




from vsg import rule


class rule_024(rule.rule):
    '''
    Instantiation rule 024 checks for positional generic and port assignments.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '024'
        self.solution = 'Rewrite positional assignment to explicit assignment.'
        self.phase = 1
        self.fixable = False  # This requires the user to map the ports and generics

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.insideInstantiation:
            if ',' in oLine.lineNoComment:
                lLine = oLine.lineNoComment.split(',')
                for sString in lLine[:-1]:
                    if '=>' not in sString:
                        self.add_violation(iLineNumber)
                        break



import re


def entity(self, dVars, oLine):
    '''
    Classifies entity declarations.

    entity identifier is
        [ port ( port_interface_list ) ;]
        { entity_declarative_item }
    [begin
        { concurrent_assertion_statement
        | passive_concurrent_procedure_call_statement
        | passive_process_statement } ]
    end [entity] [identifier] ;

    Sets the following line attributes:

      * hasEntity
      * isEntityDeclaration
      * identLevel
      * insideEntity
      * isEndEntityDeclaration

    Modifies the following variables:

      * iCurrentIndentLevel
    '''
    # Check for entity
    if re.match('^\s*entity', oLine.lineLower):
        self.hasEntity = True
        oLine.isEntityDeclaration = True
        dVars['iCurrentIndentLevel'] = 1
        oLine.indentLevel = 0
        oLine.insideEntity = True
    # Check for the end of the entity
    if re.match('^\s*end', oLine.lineLower) and not oLine.insidePortMap and not oLine.insideGenericMap and oLine.insideEntity:
        oLine.isEndEntityDeclaration = True
        oLine.indentLevel = 0
        dVars['iCurrentIndentLevel'] = 0




from vsg import rule
from vsg import check


class rule_008(rule.rule):
    '''
    Comment rule 008 checks consecutive comment lines above an "elsif" keyword
    in an if statement are aligned with the "elsif" keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'comment'
        self.identifier = '008'
        self.solution = 'Align comment with "elsif" keyword.'
        self.phase = 4

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isElseIfKeyword:
            check.indent_of_comments_above(self, oFile, iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oFile.lines[iLineNumber].indentLevel = self.dFix['violations'][iLineNumber]
            sLine = oFile.lines[iLineNumber].line.strip()
            oFile.lines[iLineNumber].update_line(' ' * self.indentSize * self.dFix['violations'][iLineNumber] + sLine)




from .rule_001 import rule_001

from .rule_003 import rule_003
from .rule_004 import rule_004
from .rule_005 import rule_005
from .rule_006 import rule_006

from .rule_008 import rule_008
from .rule_009 import rule_009
from .rule_010 import rule_010




from vsg.rules import keyword_alignment_rule


class rule_006(keyword_alignment_rule):
    '''
    Comment rule 006 ensures the alignment of "--" keywords between the process sensitivity list and the process "begin" keywords.
    '''

    def __init__(self):
        keyword_alignment_rule.__init__(self, 'comment', '006')
        self.solution = 'Inconsistent alignment of comments process declaration region.'
        self.phase = 6
        self.sKeyword = '--'
        self.sStartGroupTrigger = 'isSensitivityListEnd'
        self.sEndGroupTrigger = 'isProcessBegin'
        self.sLineTrigger = 'hasInlineComment'





from vsg.rules import single_space_after_rule


class rule_006(single_space_after_rule):
    '''
    Library rule 006 checks for a single space after the use keyword.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'library', '006', 'isLibraryUse', 'use')
        self.solution = 'Remove extra spaces after "use" keyword.'




from vsg import rule


class rule_001(rule.rule):
    '''
    Checks for line length violations.

    '''

    def __init__(self):
        rule.rule.__init__(self, 'length', '001')
        self.phase = 7
        self.solution = None
        self.fixable = False  # The user will have to fix line length violations
        self.disable = False
        self.length = 120
        self.configuration.append('length')

    def _analyze(self, oFile, oLine, iLineNumber):
        if len(oLine.line) > self.length:
            self.add_violation(iLineNumber)

    def _get_solution(self, iLineNumber):
        return 'Reduce line to less than ' + str(self.length) + ' characters'




from vsg import rule
from vsg import utils

import re


class rule_016(rule.rule):
    '''
    Generic rule 016 checks for multiple generic terms on a single line.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'generic'
        self.identifier = '016'
        self.solution = 'Move multiple generics to their own lines.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isGenericDeclaration and re.match('^.*;.*:', oLine.lineNoComment):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            for i in range(0, oFile.lines[iLineNumber].line.count(';')):
                utils.split_line_after_word(oFile, iLineNumber + i, ';')




from vsg.rules import indent_rule


class rule_016(indent_rule):
    '''
    Generate rule 016 checks the indent of the when keywords in case generate statements.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'generate', '016', 'isGenerateCaseWhen')




from vsg.rules import case_rule
from vsg import utils


class rule_018(case_rule):
    '''
    Entity rule 018 checks the case keyword has proper case in end case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'case', '018', 'isEndCaseKeyword')
        self.solution = 'Change "case" keyword to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['case'])



import re


def signal(dVars, oLine):

    if (re.match('^\s*signal\s', oLine.lineLower) or re.match('^\s*signal$', oLine.lineLower)) and \
       not oLine.insideFunction and not oLine.insideProcedure:
        oLine.isSignal = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        oLine.insideSignal = True

    if oLine.insideSignal:
        if re.match('.*;', oLine.lineNoComment):
            oLine.isEndSignal = True




from vsg.rules import single_space_after_rule


class rule_002(single_space_after_rule):
    '''
    Library rule 002 checks for a single space after the library keyword.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'library', '002', 'isLibrary', 'library')
        self.solution = 'Remove extra spaces after "library" keyword.'




from vsg import rule

import re


class rule_033(rule.rule):
    '''
    Instantiation rule 033 checks for the component keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '033'
        self.solution = 'Remove component keyword'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isInstantiationDeclaration and not oLine.isDirectInstantiationDeclaration:
            if re.match('^\s*\w+\s*:\s*component', oLine.line, flags=re.IGNORECASE):
                self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            sLine = oFile.lines[iLineNumber].line
            sLine = re.sub(r'(^\s*\w+\s*:)(\s*)component(\s+)', r'\1 ', sLine, flags=re.IGNORECASE)
            oFile.lines[iLineNumber].update_line(sLine)




from vsg.rules import indent_rule


class rule_010(indent_rule):
    '''
    Case rule 010 checks for the proper alignment of comments.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'comment', '010', 'isComment')




from vsg import rule


class rule_001(rule.rule):
    '''
    With rule 001 checks for "with" statements.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'with', '001')
        self.fixable = False
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isWithKeyword:
            self.add_violation(iLineNumber)




from vsg.rules import single_space_after_rule


class rule_008(single_space_after_rule):
    '''
    Process rule 007 checks for a single space between the "end" and "generate" keywords.
    '''

    def __init__(self):
        single_space_after_rule.__init__(self, 'generate', '008', 'isGenerateEnd', 'end')
        self.solution = 'Ensure there is only one space between the "end" and "generate" keywords.'



import re


def for_loop(dVars, oLine):
    '''
    [loop_label :] for {identifier} in {discrete_range} loop

    end loop [loop label];
    '''

    if re.match('^\s*for\s.*\sin\s.*\sloop', oLine.lineLower):
        dVars['iForLoopLevel'] += 1
        oLine.isForLoopKeyword = True
        oLine.insideForLoop = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        dVars['iCurrentIndentLevel'] += 1
    if re.match('^\s*\w+\s*:\s*for\s.*\sin\s.*\sloop', oLine.lineLower):
        dVars['iForLoopLevel'] += 1
        oLine.isForLoopKeyword = True
        oLine.insideForLoop = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        dVars['iCurrentIndentLevel'] += 1
        oLine.isForLoopLabel = True
    if re.match('^\s*end\s+loop', oLine.lineLower) and not oLine.insideWhileLoop:
        dVars['iForLoopLevel'] -= 1
        oLine.isForLoopEnd = True
        dVars['iCurrentIndentLevel'] -= 1
        oLine.indentLevel = dVars['iCurrentIndentLevel']




from vsg import rule
from vsg import fix
from vsg import check
from abc import abstractmethod


class case_rule(rule.rule):
    '''
    Checks for and fixes words case.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    sTrigger : string
       The line attribute the rule applies to.

    Attribute
    ----------

    self.phase : integer = 6
       Sets the phase the rule will run in.

    self.solution : string = None
       Instructions on how to fix the violation.
    '''

    def __init__(self, name=None, identifier=None, sTrigger=None):
        rule.rule.__init__(self, name, identifier)
        self.phase = 6
        self.solution = None
        self.sTrigger = sTrigger
        self.case = 'lower'
        # Dictionary with words to be fixed. Key is line number, value is a list of words to fix for given key.
        self.words_to_fix = {}
        self.configuration.append('case')

    @abstractmethod
    def _extract(self, oLine):
        pass

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.__dict__[self.sTrigger]:
            words = self._extract(oLine)

            if self.case == 'lower':
                check_function = check.is_lowercase
            elif self.case == 'upper':
                check_function = check.is_uppercase
            else:
                raise Exception("case option needs to be 'lower' or 'upper', detected: {self.case}")

            words_to_fix = set()
            for word in words:
                if not check_function(self, word, iLineNumber):
                    words_to_fix.add(word)

            if words_to_fix:
                self.words_to_fix[iLineNumber] = words_to_fix

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            for word in self.words_to_fix[iLineNumber]:
                if self.case == 'lower':
                    fix_function = fix.lower_case
                else:
                    fix_function = fix.upper_case

                fix_function(oFile.lines[iLineNumber], word)

    def _get_solution(self, iLineNumber):
        return self.solution + self.case + 'case.'




from vsg.rules import case_rule
from vsg import utils


class rule_005(case_rule):
    '''
    Generate rule 005 checks the label has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'generate', '005', 'isGenerateLabel')
        self.solution = 'Change label to ' + self.case + 'case'
        self.case = 'upper'

    def _extract(self, oLine):
        return utils.extract_label(oLine)




from vsg.rules import case_rule


class rule_019(case_rule):
    '''
    Port rule 019 checks the port direction has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'port', '019', 'isPortDeclaration')
        self.solution = 'Change port direction to .'

    def _extract(self, oLine):
        return [oLine.line.split(':')[1].split()[0]]




from vsg.rules import remove_spaces_before_character_rule


class rule_004(remove_spaces_before_character_rule):
    '''
    Whitespace rule 004 checks for spaces before commas.
    '''

    def __init__(self):
        remove_spaces_before_character_rule.__init__(self, 'whitespace', '004', ',')
        self.solution = 'Remove spaces before commas.'



import re


def case(self, dVars, oLine):
    '''
    [case_label :] case {expression} is
      ( when {choices} => {sequential_statement} )
      ( when {choices} => {sequential_statement} )
    end case [case_label];
    '''

    if re.match('^\s*case[\s|\(]', oLine.lineLower):
        oLine.insideCaseStatement = True
        oLine.isCaseKeyword = True
        oLine.insideCase = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        dVars['iCurrentIndentLevel'] += 2
    if re.match('^\s*\w+\s*:\s*case[\s|\(]', oLine.lineLower) and not re.match('^\s*\w+\s*:\s*case\s.*\sgenerate', oLine.lineLower):
        oLine.insideCaseStatement = True
        oLine.isCaseKeyword = True
        oLine.insideCase = True
        oLine.hasCaseLabel = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        dVars['iCurrentIndentLevel'] += 2
    if oLine.insideCase:
        if re.match('^\s*.*[\s|\)]is\s', oLine.lineLower) or re.match('^\s*.*[\s|\)]is$', oLine.lineLower):
            oLine.isCaseIsKeyword = True
    if re.match('^\s*when\s', oLine.lineLower) and not oLine.insideConcurrent and not oLine.insideSequential and oLine.insideCaseStatement:
        oLine.isCaseWhenKeyword = True
        oLine.insideCaseWhen = True
        oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
    if oLine.insideCaseWhen:
        oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
        if re.match('^\s*.*=>', oLine.line):
            oLine.isCaseWhenEnd = True
    if re.match('^\s*end\s+case', oLine.lineLower):
        oLine.isEndCaseKeyword = True
        oLine.indentLevel = dVars['iCurrentIndentLevel'] - 2
        dVars['iCurrentIndentLevel'] -= 2
    if re.match('^\s*end\s+case\s+\w+\s*;', oLine.lineLower):
        oLine.hasEndCaseLabel = True

    # Check for null statements
    if re.match('^\s*null\s*;', oLine.lineLower):
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        oLine.isCaseNull = True




from vsg import rule
from vsg import utils


class rule_012(rule.rule):
    '''
    Process rule 012 checks for the existance of the "is" keyword.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'process', '012')
        self.solution = 'Add "is" keyword after the closing parenthesis.'
        self.phase = 1

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isSensitivityListEnd and not oLine.isProcessIs:
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            iInsertIndex = oLine.line.rfind(')')
            oLine.update_line(oLine.line[:iInsertIndex + 1] + ' is ' + oLine.line[iInsertIndex + 1:])
            oLine.isProcessIs = True
            search_for_and_remove_extraneous_is(oFile, iLineNumber)


def search_for_and_remove_extraneous_is(oFile, iLineNumber):
    '''
    Looks for an is keyword that is not on the same line as the closing parenthesis and removes it.
    '''
    iIndex = iLineNumber
    while True:
        iIndex += 1
        oLine = oFile.lines[iIndex]
        if oLine.isProcessIs:
            utils.clear_keyword_from_line(oLine, 'is')
            oLine.isProcessIs = False
        if oLine.isProcessBegin:
            break




from vsg.rules import line_above_rule


class rule_010(line_above_rule):
    '''
    Type 010 checks for a blank line above the "type" declaration.
    '''

    def __init__(self):
        line_above_rule.__init__(self, 'type', '010', 'isTypeKeyword')




from vsg.rules import case_rule
from vsg import utils


class rule_004(case_rule):
    '''
    Constant rule 004 checks the constant names have proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'constant', '004', 'isConstant')
        self.solution = 'Change constant identifiers name to '

    def _extract(self, oLine):
        return utils.extract_class_identifier_list(oLine)




from vsg import rule


class rule_012(rule.rule):
    '''Whitespace rule 012 checks the number of consecutive blank lines.'''

    def __init__(self):
        rule.rule.__init__(self, 'whitespace', '012')
        self.phase = 3
        self.numBlankLines = 1
        self.solution = 'Remove all but ' + str(self.numBlankLines) + ' blank lines below.'

        self.iNumBlankLines = 0
        self.fFoundFirstBlank = False
        self.iErrorLine = 0

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isBlank and not self.fFoundFirstBlank:
            self.fFoundFirstBlank = True
            self.iErrorLine = iLineNumber
        if oLine.isBlank:
            self.iNumBlankLines += 1
        if self.iNumBlankLines > self.numBlankLines:
            self.add_violation(self.iErrorLine)
            self.dFix['violations'][self.iErrorLine] = self.iNumBlankLines - self.numBlankLines
        if not oLine.isBlank:
            self.fFoundFirstBlank = False
            self.iErrorLine = 0
            self.iNumBlankLines = 0

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            del oFile.lines[iLineNumber:iLineNumber + self.dFix['violations'][iLineNumber]]




from vsg import rule

import re


class rule_001(rule.rule):
    '''Whitespace rule 001 checks spaces at the end of lines.'''

    def __init__(self):
        rule.rule.__init__(self, 'whitespace', '001')
        self.phase = 2
        self.solution = 'Remove trailing whitespace.'

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.line.endswith(' '):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub(r'\s+$', '', oLine.line))




from vsg import rule

import re


class rule_005(rule.rule):
    '''Whitespace rule 005 checks for spaces after an open parenthesis.'''

    def __init__(self):
        rule.rule.__init__(self, 'whitespace', '005')
        self.phase = 2
        self.solution = 'Remove spaces after open (.'

    def _analyze(self, oFile, oLine, iLineNumber):
        sLine = oLine.lineNoComment
        if re.match('^.*\(\s+', sLine) and not re.match('^.*\(\s+[0-9]', sLine):
            self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            iCommentIndex = oLine.line.find('--')
            if iCommentIndex == -1:
                oLine.update_line(re.sub(r'\((\s+)([A-Za-z_\(])', r'(\2', oLine.line))
            else:
                sLine = oLine.line[:iCommentIndex]
                sLine = re.sub(r'\((\s+)([A-Za-z_\(])', r'(\2', sLine)
                sLine = sLine + oLine.line[iCommentIndex:]
                oLine.update_line(sLine)




from vsg import rule
from vsg import utils


class rule_016(rule.rule):
    '''
    Signal rule 016 checks a signal declaration is on a single line.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'signal', '016')
        self.solution = 'ensure signal declaration is on a single line.'
        self.phase = 1

    def _pre_analyze(self):
        self.sFullLine = ''
        self.iFailureLine = None

    def _analyze(self, oFile, oLine, iLineNumber):
        if oLine.isSignal:
            self.sFullLine = ''
            self.iFailureLine = iLineNumber
        if oLine.insideSignal:
            self.sFullLine += oLine.line
        if oLine.isEndSignal:
            if not oLine.isSignal:
                self.add_violation(self.iFailureLine)
                self.dFix['violations'][self.iFailureLine] = {}
                self.dFix['violations'][self.iFailureLine]['endLine'] = iLineNumber
                self.dFix['violations'][self.iFailureLine]['line'] = self.sFullLine

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations[::-1]:
            dViolation = self.dFix['violations'][iLineNumber]
            utils.remove_lines(oFile, iLineNumber, dViolation['endLine'])
            utils.insert_line(oFile, iLineNumber)
            oLine = oFile.lines[iLineNumber]
            oLine.isSignal = True
            oLine.insideSignal = True
            oLine.isEndSignal = True
            oLine.isBlank = False
            oLine.update_line(dViolation['line'])
            utils.update_comment_line_attributes(oLine)



from setuptools import setup
from setuptools import find_packages


from vsg import version


def readme():
    with open('README.rst') as f:
        return f.read()


setup(
  name='vsg',
  version=str(version.version),
  description='VHDL Style Guide',
  long_description=readme(),
  classifiers=[
      'Development Status :: 4 - Beta',
      'Environment :: Console',
      'Programming Language :: Python :: 3',
      'Intended Audience :: End Users/Desktop',
      'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
      'Natural Language :: English',
      'Operating System :: POSIX :: Linux',
      'Topic :: Software Development :: Quality Assurance',
      'Topic :: Text Processing :: General',
  ],
  url='https://github.com/jeremiah-c-leary/vhdl-style-guide',
  download_url='https://github.com/jeremiah-c-leary/vhdl-style-guide',
  author='Jeremiah C Leary',
  author_email='jeremiah.c.leary@gmail.com',
  license='GNU General Public License',
  packages=find_packages(),
  zip_safe=False,
  test_suite='nose.collector',
  tests_require=['nose'],
  keywords=['vhdl', 'style', 'beautify', 'guide', 'lint'],
  install_requires=[
    'PyYAML'
  ],
  entry_points={
    'console_scripts': [
      'vsg = vsg.__main__:main'
    ]
  }
)




from vsg.rules import line_below_rule


class rule_011(line_below_rule):
    '''
    Process rule 010 checks for a blank line after the "end process" keywords.
    '''

    def __init__(self):
        line_below_rule.__init__(self, 'process', '011', 'isEndProcess')




from vsg.rules import remove_blank_lines_above_rule


class rule_001(remove_blank_lines_above_rule):
    '''
    Port rule 001 checks for a blank line above the port keyword.
    '''

    def __init__(self):
        remove_blank_lines_above_rule.__init__(self, 'port', '001', 'isPortKeyword')
        self.solution = 'Remove blank lines above "port" keyword.'




from vsg.rules import case_rule
from vsg import utils


class rule_010(case_rule):
    '''
    Generate rule 010 checks the generate keyword in the "end generate" has the proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'generate', '010', 'isGenerateEnd')
        self.solution = 'Change "generate" to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['generate'])




from vsg.rules import remove_blank_lines_below_rule


class rule_022(remove_blank_lines_below_rule):
    '''
    Port rule 022 checks for a blank line below the port keyword.
    '''

    def __init__(self):
        remove_blank_lines_below_rule.__init__(self, 'port', '022', 'isPortKeyword')
        self.solution = 'Remove blank lines below "port" keyword.'




from vsg.rules import case_rule


class rule_009(case_rule):
    '''
    Instantiation rule 009 checks the entity name has proper case in the instantiation declaration line.
    '''

    def __init__(self):
        case_rule.__init__(self, 'instantiation', '009', 'isInstantiationDeclaration')
        self.case = 'upper'
        self.solution = 'Change entity name to '

    def _extract(self, oLine):
        return [oLine.line.split(':')[1].lstrip().split()[0]]




from vsg import rule
from vsg import check
from vsg import fix
from vsg import line


class rule_029(rule.rule):
    '''
    Instantiation rule 029 ensures the alignment of comments in an instantiation.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '029'
        self.solution = 'Inconsistent alignment of comments in instantiation.'
        self.phase = 6
        self.lGroup = []
        self.fGroupFound = False
        self.iStartGroupIndex = None

    def _analyze(self, oFile, oLine, iLineNumber):
            self.fGroupFound, self.iStartGroupIndex = search_for_group(self.fGroupFound, oLine, self.iStartGroupIndex, iLineNumber)
            if oLine.isInstantiationPortEnd:
                self.lGroup.append(oLine)
                self.fGroupFound = False
                check.keyword_alignment(self, self.iStartGroupIndex, '--', self.lGroup)
                self.lGroup = []
                self.iStartGroupIndex = None
            store_lines_for_group(self.fGroupFound, oLine, self.lGroup)

    def _fix_violations(self, oFile):
        fix.keyword_alignment(self, oFile)


def search_for_group(fGroupFound, oLine, iStartGroupIndex, iLineNumber):
    if not fGroupFound and oLine.insideInstantiation:
        return True, iLineNumber
    return fGroupFound, iStartGroupIndex


def store_lines_for_group(fGroupFound, oLine, lGroup):
    if fGroupFound:
        if oLine.hasComment and not oLine.isComment:
            lGroup.append(oLine)
        else:
            lGroup.append(line.line('Removed line'))




from vsg.rules import case_rule
from vsg import utils


class rule_004(case_rule):
    '''
    Function rule 004 checks the "begin" keyword has proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'function', '004', 'isFunctionBegin')
        self.solution = 'Change "begin" keyword to '

    def _extract(self, oLine):
        return utils.extract_words(oLine, ['begin'])




from vsg.rules import indent_rule


class rule_012(indent_rule):
    '''
    Type rule 012 checks the indent of record elements.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'type', '012', 'insideTypeRecord')




from vsg.rules import case_rule
from vsg import utils


class rule_007(case_rule):
    '''
    Generic rule 007 checks generic names have proper case.
    '''

    def __init__(self):
        case_rule.__init__(self, 'generic', '007', 'isGenericDeclaration')
        self.case = 'upper'
        self.solution = 'Change generic name to '

    def _extract(self, oLine):
        return utils.extract_generics(oLine)




from vsg.rules import line_above_rule


class rule_007(line_above_rule):
    '''
    Case rule 007 ensures a blank line exists before the "case" keyword.
    '''

    def __init__(self):
        line_above_rule.__init__(self, 'case', '007', 'isCaseKeyword', 'isComment')




from vsg.rules import line_above_rule


class rule_006(line_above_rule):
    '''
    Function rule 006 enforces a blank line above the "function" keyword.
    '''

    def __init__(self):
        line_above_rule.__init__(self, 'function', '006', 'isFunctionKeyword')
